/*
    Web2OSC.java -- routes incoming data from Web clients to MAX.

    Copyright (C) 2001 John Paul Young. All Rights Reserved.

    This file is part of Web2OSC.

    Web2OSC is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    Web2OSC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Web2OSC; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.net.*;
import java.io.*;

public class Web2OSC {

  protected static int defaultPort = 7777;
  protected static int defaultBufferLength = 128;
  
  public static void main(String[] args) {

    String hostname;
    int port;
    int len;

    ServerSocket ss;
    Socket sock;

    try {
      hostname = args[0];
    }
    catch (Exception e) {
      hostname = "localhost";
    }
    try {
      port = Integer.parseInt(args[1]);
    }
    catch (Exception e) {
      port = defaultPort;
    }
    try {
      len = Integer.parseInt(args[2]);
    }
    catch (Exception e) {
      len = defaultBufferLength;
    }

    try {
      ss = new ServerSocket(port);
      System.out.println("Listening for TCP connections on port " + ss.getLocalPort());
      
      DatagramSocket ds = new DatagramSocket();
      byte[] buffer = new byte[len];
      InetAddress ia = InetAddress.getByName(hostname);
      DatagramPacket dp = new DatagramPacket(buffer, buffer.length, ia, port);

      int id = 0;
      String cmd = "", arg = "0";
      StringBuffer buf;

      while (true) {
        sock = ss.accept();
        System.out.println("Connection from " + sock);
        InputThread it = new InputThread(sock.getInputStream());
        buf = new StringBuffer();
        buf.append(it.read());
        
        Character c1 = new Character(buf.charAt(0));
        Character c2 = new Character(buf.charAt(1));
        id = c1.hashCode();
        arg = String.valueOf(c2.hashCode());
        
        // wait for Thread to finish 
        try {
        it.join();
        }
        catch (InterruptedException e) {
          System.err.println(e);
        }
      toOSC(ds, dp, id, cmd, arg);
      }
    }
    catch (IOException e) {
      System.err.println(e);
    }
  }

  public static void toOSC(DatagramSocket ds, DatagramPacket dp, int id, String cmd, String arg) {

    String test = "#bundle             000100020003";

    byte[] data = new byte[test.length()];
    data = test.getBytes();
    data[7] = data[8] = data[9] = data[10] = data[11] = data[12] = data[13] = data[14] = 0;
    data[15] = 1;
    data[16] = data[17] = data[18] = 0;
    data[19] = 12;
    data[20] = data[21] = data[22] = 0;
    data[23] = 0;
    data[24] = data[25] = data[26] = 0;
    data[27] = Byte.parseByte(String.valueOf(id));
    data[28] = data[29] = data[30] = 0;
    data[31] = Byte.parseByte(arg);
    test = new String(data);
//  System.out.println("Sent: " + test);
      
    dp.setData(data);

    try {
      ds.send(dp);
    }
    catch (IOException e) {
      System.err.println(e);
    }  
  }
}

class InputThread extends Thread {
  
  InputStream is;
  
  public InputThread(InputStream is) {
    this.is = is;
  }

  public String read() {
   
    String temp = "";
    StringBuffer tempbuf = new StringBuffer();
    
    try {
      while (true) {
        int i = is.read();
        if (i == -1) break;
        char c = (char) i;
// System.out.print(c);
        tempbuf.append(c);
      }
    }
    catch (IOException e) {
      System.err.println(e);
    }
    temp = new String(tempbuf);
    return temp;
  }
}
