/*
    rtIPTime.java -- routes IP and Time data from a Web page to MAX.

    Copyright (C) 2001 John Paul Young. All Rights Reserved.

    This file is part of Web2OSC.

    Web2OSC is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    Web2OSC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Web2OSC; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.net.*;
import java.io.*;
import java.util.*;

public class rtIPTime {

  protected static int defaultPort = 7778;
  protected static int defaultBufferLength = 128;
  
  public static void main(String[] args) {

    String hostname;
    int port;
    int len;

    ServerSocket ss;
    Socket sock;

    try {
      hostname = args[0];
    }
    catch (Exception e) {
      hostname = "localhost";
    }
    try {
      port = Integer.parseInt(args[1]);
    }
    catch (Exception e) {
      port = defaultPort;
    }
    try {
      len = Integer.parseInt(args[2]);
    }
    catch (Exception e) {
      len = defaultBufferLength;
    }

    try {
      ss = new ServerSocket(port);
      System.out.println("Listening for TCP connections on port " + ss.getLocalPort());
// System.err.println("Listening for TCP connections on port " + ss.getLocalPort());
      
      DatagramSocket ds = new DatagramSocket();
      byte[] buffer = new byte[len];
      InetAddress ia = InetAddress.getByName(hostname);
      DatagramPacket dp = new DatagramPacket(buffer, buffer.length, ia, port);

      int hour = 0;
      int min = 0;
      int sec = 0;
      
      Calendar nowCal;
      Date nowDate;
      
      while (true) {
        sock = ss.accept();
        
        InetAddress visitor = sock.getInetAddress();
        byte[] ip = visitor.getAddress();

        nowCal = Calendar.getInstance();
        nowDate = nowCal.getTime();
        hour = nowCal.get(Calendar.HOUR_OF_DAY);
        min = nowCal.get(Calendar.MINUTE);
        sec = nowCal.get(Calendar.SECOND);

System.out.println("IP: " + ip[0] + "." + ip[1] + "." + ip[2] + "." + ip[3] + " at " +
                   "Hour: " + hour + " Min: " + min + " Sec: " + sec);

// System.out.println("On " + nowDate.toString());
// System.out.println("Connection from " + sock);
        
        toOSC(ds, dp, ip[0], ip[1], ip[2], ip[3], hour, min, sec);
      }
    }
    catch (IOException e) {
      System.err.println(e);
System.out.println(e);    
    }
  }

  public static void toOSC(DatagramSocket ds, DatagramPacket dp,
                           byte ip0, byte ip1, byte ip2, byte ip3, int hour, int min, int sec) {

     String msg = "#bundle             00010002000300040005000600070008";

     byte[] data = new byte[msg.length()];
     data = msg.getBytes();
     data[7] = data[8] = data[9] = data[10] = data[11] = data[12] = data[13] = data[14] = 0;
     data[15] = 1;
     data[16] = data[17] = data[18] = 0;
     data[19] = 32;
     data[20] = data[21] = data[22] = 0;
     data[23] = 0;
     data[24] = data[25] = data[26] = 0;
     data[27] = ip0; // value #2
     data[28] = data[29] = data[30] = 0;
     data[31] = ip1; // value #3
     data[32] = data[33] = data[34] = 0;
     data[35] = ip2; // value #4
     data[36] = data[37] = data[38] = 0;
     data[39] = ip3; // value #5
     data[40] = data[41] = data[42] = 0;
     data[43] = (byte) hour; // value #6
     data[44] = data[45] = data[46] = 0;
     data[47] = (byte) min; // value #7
     data[48] = data[49] = data[50] = 0;
     data[51] = (byte) sec; // value #8

// msg = new String(data);
// System.out.println("Sent: " + msg);
      
    dp.setData(data);

    try {
      ds.send(dp);
    }
    catch (IOException e) {
      System.err.println(e);
    }  
  }
}

